;Http interface driver for lookout

;;;;;;;;;;; TEMPORARY FOR DEBUG

;all this code will be rewritten to work with context

(define http-output (list #f)) ; current output queue
(define (http-post-event . command) 
  (append! http-output (list command)))


(define (make-http-id-generator)
  (let ((counter 0))
    (lambda ()
      (begin-1
	(string-append "w" (number->string counter))
	(set! counter (+ counter 1))))))
(define http-id-generator (make-http-id-generator)) ;current id generator


(define http-dialogs '()) ;current active dialogs

(define (http-cmd-get url)
  (format #t "getting url=~S~%" url)
  (let ((dialog (cond-cdr (assq (sure-symbol url) http-dialogs))))
    (if (not dialog)
      (let ((dlg (init-dialog (list url))))
	(and dlg
	     (set! http-dialogs (alist-set (sure-symbol url) dlg http-dialogs))))
      #f ;TODO: reproduce initial commands for output queue
      )))

(define http-widgets '()) ;current active widgets, our entry point for callbacks


;;;;;;;;;;; TEMPORARY FOR DEBUG

(define (make-http-widget-dispatcher)
  (let ((attributes '()))
    (lambda (id command . args)
      (case command
	((set) (set! attributes (alist-set (sure-symbol (car args))
					   (cadr args)
					   attributes))
	       (http-post-event 'action "set"
				'widget-id id
				'attr (car args)
				'value (cadr args)))
	((get) (cond-cdr (assq (sure-symbol (car args)) attributes)))
	(else
	  (error "unknown operation for http widget"))))))

(define (make-http-widget type . rest)
  (let ((dispatcher (if (null? rest) (make-http-widget-dispatcher) (car rest))))
    (lambda (events-holder)
      (let ((id #f)
	    (events events-holder) ;events dispatcher
	    (pending-requests (list #f)))
      (lambda (command . args)
	(case command
	  ((get-http-id) id)
	  ((new) (set! id (http-id-generator))
		 (http-post-event 'action "new"
				  'widget-id id
				  'type type
				  'parent (if (procedure? (car args))
					    ((car args) 'get-http-id)
					    "")))
	  ((post-new)
		 (for-each (lambda (x)
		   		(apply dispatcher widget (car x) (cdr x))) ; apply delayed requests
			     (cdr pending-requests)))
	  (else  (if id
		   (apply dispatcher id command args)
		   (begin (append! pending-requests (list (cons command args)))
			  #f)))))))))


(define *server-socket* #f)
(define *socket-name* "/tmp/alterator-sock")

(define (delete-http-application app)
  (close-port *server-socket*)
  (delete-file *socket-name*))

(define (make-http-application cmdline handler)
  (write-line "creating server side socket")
  (set! *server-socket* (socket PF_UNIX SOCK_STREAM 0))
  (bind *server-socket* AF_UNIX *socket-name*)
  (listen *server-socket* 10)
  (chmod *socket-name* #o666))


(define (exec-http-application)
  (write-line "listen for connections ...")
  (let listen-loop ((c (car (accept *server-socket*))))
    (write-line "accepting connection and process commands")
    (let command-loop ((cmd (read c)))
      (format #t "readed:~S~%" cmd)
      (and (not (eof-object? cmd))
	   (let ((action (cond-cdr (command-arg-ref cmd 'action))))
	     (and action
		  (case (sure-symbol action)
		    ((get) (write-line "this is a get action")
			   (http-cmd-get (cond-cdr (command-arg-ref cmd 'url)))
			   (write (cdr http-output) c)
			   (set! http-output (list #f)))
		    ((event) (write-line "this is a event notification"))))
	     (command-loop (read c)))))
    (write-line "go to next processing")
    (listen-loop (car (accept *server-socket*)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;driver description
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(declare-widget button (make-http-widget "button") text)
(declare-widget edit (make-http-widget "edit") text)
(declare-widget hbox (make-http-widget "hbox"))
(declare-widget vbox (make-http-widget "vbox"))

;wrapper over tab-panel
;insert widgets and generate also tab-panel attribute
(define (tab tab-id tab-label . widgets)
  (let ((vb (id tab-id (apply vbox widgets))))
   `(,vb ,(tab-panel tab-label (cadr vb)))))

(declare-application-constructor make-http-application)
(declare-application-destructor delete-http-application)
(declare-application-executor exec-http-application)

(declare-main-widget (make-http-widget "vbox"))

