(use-modules (srfi srfi-1) (srfi srfi-13); to use in modules
	     
	     (ice-9 syncase)
	     
	     (alterator str)
	     (alterator help)
	     (alterator gettext)
	     (alterator glob)
	     (alterator algo)
	     (alterator sandbox)
	     (alterator command)
	     
	     (alterator lookout common)
	     (alterator lookout woo)
	     (alterator lookout uri)
	     (alterator lookout context)
	     (alterator lookout ids)
	     (alterator lookout globals)
	     (alterator lookout events)
	     (alterator lookout goto)
	     (alterator lookout mapper)
	     (alterator lookout widgets)
	     (alterator lookout engine))

;load common definitions for all drivers
(load (string-append datadir "lookout-driver.scm"))

(define id set-id) ; guile suxx has same built-in function
(define include include-widget) ; guile suxx, ice-9 syncase also export include method

(define (dbus-handler sender destination interface dmember . args)
  (let ((current-dialog (from-context 'dialog)))
    (and current-dialog
	 (current-dialog 'call on-dbus
			 sender
			 destination
			 interface
			 dmember
			 args))))

(define-syntax declare-application-constructor
  (syntax-rules()
    ((_ func) (define lookout-make-application func))))

(define-syntax declare-application-destructor
  (syntax-rules()
    ((_ func) (define lookout-delete-application func))))

(define-syntax declare-application-executor
  (syntax-rules()
    ((_ func) (define lookout-exec-application func))))

(define-syntax with-application-driver
  (syntax-rules()
    ((_ drv-path instruction instructions ...)
     (begin
       (load drv-path)
       (let ((app (lookout-make-application (command-line) dbus-handler)))
	 instruction
	 instructions ...
	 (lookout-delete-application app))))))

;;;;;;;;;;;;wait-message support

(define (with-splash-message thunk)
  (let* ((splash (splashscreen))
	 (sp (cadr splash)))
    (with-changed-context 'splash sp
			  (dynamic-wind
			    (lambda()
			      ((caddr splash) #f)) ;initialize screen
			    thunk
			    (lambda()
			      (sp 'delete))))))

(define (splash-message msg)
  ((from-context 'splash) 'set 'message msg))

(define (splash-image img)
  ((from-context 'splash) pixmap img))


;run main dialog
(define (lookout-main filename)
 (with-mapper (load-mapper filename)
	      (lookout-exec-application)))

(define (lookout filename . rest)
  (let ((driver (if (null? rest) "qt" (car rest))))
    (with-application-driver (string-append datadir "lookout-" driver ".scm")
			     (lookout-main filename))))


