(define-module (alterator lookout engine)
	       :use-module (ice-9 syncase)
	       :use-module (alterator algo)
	       :use-module (alterator command)
	       :use-module (alterator lookout uri)
	       :use-module (alterator lookout context)
	       :use-module (alterator lookout globals)
	       :use-module (alterator lookout mapper)
	       :use-module (alterator lookout widgets)
	       :use-module (alterator lookout goto)
	       :use-module (alterator lookout ids)
	       :export (declare-main-widget
	       		lookout-main-widget
			
	       		run-dialog
			run-widget
			
			init-dialog
			init-widget
			
			end-dialog

			load-url
			include-as-is
			include-widget
			
			with-dialog
		
			make-relative-size
			relative-width
			relative-height))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define-syntax with-dialog
  (syntax-rules ()
		((_ dlg instruction instructions ...)
		 (with-changed-context 'dialog dlg instruction instructions ...))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define lookout-main-widget #f)

(define-macro (declare-main-widget func)
	      `(set! lookout-main-widget ,func))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;create dialog object: widget with support of continuation and main subwidget
(define (make-dialog widget-constructor parent)
  (let ((widget (make-widget widget-constructor))
	(return-value #f))
    (widget 'new parent) ; all dialogs are without parents
    (lambda (command . args)
      (case command
	((start)  (set! return-value #f)
		  (widget 'start)
		  return-value)
	((stop)   (widget 'stop)
		  (if (not (null? args))
		    (set! return-value (car args))))
	(else (apply widget command args))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;create dialog with appropriate widgets
;Note: we use widget loader function ,cause we want to evaluate all expressions
;in the right context created in this function
(define (init-widget widget-loader . args)
  (and (procedure? widget-loader)
       (let* ((parent (if (null? args) (from-context 'dialog) (car args)))
	      (dlg (make-dialog lookout-main-widget parent)))
	    (with-goto-frame dlg
		 (with-dialog dlg
		      (with-ids (make-ids-holder)
			    (widget-parse-args dlg (widget-loader))
			    ((from-context 'dialog) 'set-context (fluid-ref lookout-context)))))
	    dlg)))

;run widget with appropriate widgets
(define (run-widget widget-loader . args)
  (let ((dlg (apply init-widget widget-loader args)))
    (and (procedure? dlg)
	 (with-dialog dlg
		      (begin-1
			(dlg 'start)
			(dlg 'delete))))))

;resolve uri, and create new dialog
;return created dialog class
(define (init-dialog uri . args)
    (with-globals (make-globals '())
      (init-widget (apply load-url uri)
                  (if (null? args) (from-context 'dialog) (car args)))))

;resolve uri, and run new dialog
;return dialog's return code
(define (run-dialog uri . args)
  (let ((dlg (apply init-dialog uri args)))
    (and (procedure? dlg)
	 (with-dialog dlg
		      (begin-1
			(dlg 'start)
			(dlg 'delete))))))

;exit current dialog modal loop
(define (end-dialog . args) (apply (from-context 'dialog) 'stop args))

;load widget using appropriate url
;return loader function: to run later in appropriate context
(define (load-url . url)
  (let ((path (lookout-convert-uri url)))
    (and path
	 (lambda()
	       (command-for-each (lambda(name value) (set-global! name value)) path)
	       (load-widgets (car path))))))

(define (include-as-is url) ((load-url url)))
(define include-widget load-widgets)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-relative-size value name source-widget)
  `(attr ,name
	 ,(quotient (* value (source-widget 'get name))
		    100)))

(define-syntax relative-width
  (syntax-rules()
    ((_ value) (relative-width value (from-context 'dialog)))
    ((_ value source-widget)
     (lambda() (make-relative-size value 'width source-widget)))))

(define-syntax relative-height
  (syntax-rules()
    ((_ value) (relative-height value (from-context 'dialog)))
    ((_ value source-widget)
     (lambda() (make-relative-size value 'height source-widget)))))

